<?php
// $Id: pifr.cron.inc,v 1.8 2008/10/31 00:25:15 boombatower Exp $
/**
 * @file
 * Automatically review patches attached to issues.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Check if cron should run for the specified role and if cron should run then
 * updates the last run variable to the current time.
 *
 * @param string $role Role string: test_master, or project.
 * @return boolean Cron should run.
 */
function pifr_cron_check($role) {
  $last = variable_get('pifr_' . $role . '_cron_last', 0);
  $delay = variable_get('pifr_' . $role . '_cron_delay', 5);
  $run = ($last + ($delay * 60) <= time());

  if ($run) {
    variable_set('pifr_' . $role . '_cron_last', time());
  }
  return $run;
}

/**
 * Find any free slaves and send them a file to review.
 */
function pifr_send_free_slaves() {
  $slaves = pifr_server_free_slaves();
  foreach ($slaves as $slave) {
    // Free slave found, send next file in queue.
    $file = pifr_file_next();
    if ($file) {
      pifr_send_to_slave($slave, $file);
    }
    else {
      break; // No files in queue.
    }
  }
}

/**
 * Send the results that have not yet been sent.
 */
function pifr_send_results() {
  $servers = pifr_server_get_type(PIFR_SERVER_PROJECT);
  $last = variable_get('pifr_result_last', 0);
  variable_set('pifr_result_file_id', time());
  $self = pifr_server_get_type(PIFR_SERVER_TEST_MASTER);

  foreach ($servers as $server) {
    // XXX PIFT specific server array.
    $server_param = array('url' => url('', array('absolute' => TRUE)) . 'xmlrpc.php', 'server_key' => $self->server_key);

    $batches = pifr_get_results($server->server_id, $last);

    foreach ($batches as $batch) {
      // XXX PIFT specific.
      $response = xmlrpc($server->url . 'xmlrpc.php', 'pift.test.results', $server_param, $batch);
//      $response = xmlrpc($server->url . 'xmlrpc.php', 'pifr.batch.result', $self->server_key, $batch);

      if ($response === FALSE) {
        watchdog('pifr', 'Failed to send batch of results to project server @server_id: @message',
                          array('@server_id' => $server->server_id, '@message' => xmlrpc_error_msg()), WATCHDOG_ERROR);
      }
      else if (!empty($response)) {
//      else if ($response != PIFR_RESPONSE_ACCEPTED) {
        watchdog('pifr', 'Failed to send batch of results to project server @server_id.',
                          array('@server_id' => $server->server_id), WATCHDOG_ERROR);
      }
    }
  }
}

/**
 * Get the results that have not yet been sent to the specified server.
 *
 * @param integer $server_id Server ID.
 * @param integer $last Last timestamp results were sent.
 * @return array List of batches to be sent.
 */
function pifr_get_results($server_id, $last) {
  require_once drupal_get_path('module', 'pifr') . '/pifr.pages.inc';

  $batches = array();
  $batch = array();

  // Get results since last file ID sent.
  $results = db_query('SELECT *
                       FROM {pifr_file}
                       WHERE server_id = %d
                       AND last_tested > %d
                       AND status > %d
                       ORDER BY server_id, nid, cid', $server_id, $last, PIFR_FILE_ERROR);

  while ($result = db_fetch_object($results)) {
    if (count($batch) == PIFR_MAX_FILE_COUNT) {
      $batches[] = $batch;
      $batch = array();
    }

    $file = array();
    $file['server_file_id'] = $result->server_file_id;
    $file['nid'] = $result->nid;
    $file['cid'] = $result->cid;
    $file['filename'] = $result->filename;
    $file['pass'] = ($result->status == PIFR_FILE_TEST_PASS);
    $file['summary'] = pifr_file_get_result($result) . ' ' .
                       l('Detailed results', url('pifr/file/' . $server_id . '/' . $result->filename, array('absolute' => TRUE)));

    $batch[] = pifr_convert_batch_result_file($file);
  }

  if (count($batch)) {
    $batches[] = $batch;
  }

  return $batches;
}

/**
 * Convert PIFR format.
 *
 * @param array $file File data result array in PIFR format.
 * @return array File result data array.
 */
function pifr_convert_batch_result_file($file) {
  $file['ftid'] = $file['server_file_id'];
  $file['status'] = ($file['pass'] == PIFR_FILE_TEST_PASS ? 1 : 2);
  $file['display_data'] = $file['summary'];

  unset($file['server_file_id']);
  unset($file['nid']);
  unset($file['cid']);
  unset($file['filename']);
  unset($file['pass']);
  unset($file['summary']);
  return $file;
}
